import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class Debugger implements Runnable {
    PictureCanvas P;
    boolean HALT;
    int CHOICE;

    public Debugger() {
	HALT=true;
	CHOICE=0;
    }

    public Debugger(PictureCanvas PP) {
	HALT=true;
	this.P=PP;
    }

    /**Main routine*/
    public void run() {
	CHOICE=P.DEBUG.mode;
	if(CHOICE==0) cayleyMenger();
	if(CHOICE==1) functions();
	if(CHOICE==2) cubeTransform();
	if(CHOICE==3) polynomialDilate();
	if(CHOICE==4) polynomialRotate();
	if(CHOICE==5) polynomialReflect();
	if(CHOICE==6) signEvaluator();
	if(CHOICE==7) misc();
    }


    /**Here are some routines which generate random inputs*/

    public int random(int n) {
	double x=Math.random();
	int m=(int)(Math.floor(n*x));
	return(m);
    }

    public int[] random5(int n) {
	int[] a=new int[5];
	for(int i=0;i<5;++i) a[i]=random(n);
	return(a);
    }

    public int[] random6(int n) {
	int[] a=new int[6];
	for(int i=0;i<6;++i) a[i]=random(n);
	return(a);
    }

    public Vector randomVector() {
	Vector V=new Vector();
	for(int i=0;i<3;++i) V.x[i]=Math.random();
	return(V);
    }

    /**printout routines*/

    public static void printout(int[] t) {
	for(int i=0;i<t.length;++i) System.out.print(t[i]+" ");
	    System.out.println("");
    }

    public static void printout(double[] t) {
	for(int i=0;i<t.length;++i) System.out.print(t[i]+"  ");
	    System.out.println("");
    }

    /**This routine checks that we really have computed the
       Cayley-Menger determinant correctly*/

    public void cayleyMenger() {

	Vector V0=randomVector();
	Vector V1=randomVector();
	Vector V2=randomVector();
	Vector V3=randomVector();

	Vector[] V={V0,V1,V2,V3};
	/**computes the volume*/
	Vector[] W=new Vector[3];
	for(int i=0;i<3;++i) W[i]=Vector.minus(V[i],V[3]);
	double vol=(Vector.dot(Vector.cross(W[0],W[1]),W[2]))/6;
	System.out.println("volume "+vol);


	double d1=288*vol*vol;
	System.out.println("288 vol^2  "+d1);

	double[][] D=new double[4][4];
	for(int i=0;i<4;++i) {
	    for(int j=0;j<4;++j) {
		D[i][j]=Vector.dist(V[i],V[j]);
	    }
	}
	Matrix m=CayleyMenger.cm(D);
	double d2=m.det();
	System.out.println("cm det     "+d2);

    }


    /**This routine compares the two versions of the CM functions,
       and also their partial derivatives*/

    public void functions() {
	double[] b=new double[6];
	for(int i=0;i<6;++i) b[i]=Math.floor(10*Math.random());
	int[] a=new int[6];
	for(int i=0;i<6;++i) a[i]=(int)(b[i]);
	double tol=.000000001;
       Poly6 P=DataCM.f();
       BigInteger a2=P.evaluate(a);
       System.out.println("cayley menger");
       System.out.println(a2.toString());

       System.out.println("derivatives");
       for(int k=0;k<6;++k) {
	   double c1=CayleyMenger.df(k,b);
           P=DataCM.df(k);
           BigInteger c2=P.evaluate(a); 
	   double c3=differenceQuotient(k,b,tol);
	   System.out.println("partial derivative "+k); 
	   System.out.print("floating point:        ");
           System.out.println(c1);
	   System.out.print("Poly6 class:           ");
           System.out.println(c2.toString());
	   System.out.print("10^(-9) diff quotient: ");
	   System.out.println(c3);
       }

       int[] q={1,1,1,1,1,1};
       P=DataCM.df(q);
       double c1=CayleyMenger.df(q,b);   
       BigInteger c2=P.evaluate(a); 
       System.out.println("D111111"); 
       System.out.println(c1);
       System.out.println(c2.toString());

    }


    public double differenceQuotient(int k,double[] b,double tol) {
	double[] bb=new double[6];
	for(int i=0;i<6;++i) bb[i]=b[i];
	bb[k]=b[k]+tol;
	double x0=CayleyMenger.f(b);
	double x1=CayleyMenger.f(bb);
	double y=(x1-x0)/tol;
	return(y);
    }



    /**This checks the cube transform on random inputs*/

    public void cubeTransform() {
	Poly6 P=DataCM.f();
	int[][] b=ModuliSpace.simplex3(0);
	Poly5 S=P.toCube(b);
	int[] x0=random5(10);
	int[] x3=toCube(x0,b);
	BigInteger y0=S.evaluate(x0);
	BigInteger y3=P.evaluate(x3);
	System.out.println(y0.toString());
	System.out.println(y3.toString());
    }


    public static int[] toCube(int[] x0,int[][] b) {
	int[] x1=cubeToSimplex(x0);
	int[] x2={1-x1[0],x1[0]-x1[1],x1[1]-x1[2],x1[2]-x1[3],x1[3]-x1[4],x1[4]};
	int[] x3=ModuliSpace2.map(b,x2);
	return(x3);
    }

    public static int[] cubeToSimplex(int[] t) {
	int[] u={1,1,1,1,1,0};
	u[0]=t[0];
	for(int i=1;i<5;++i) u[i]=u[i-1]*t[i];
	return(u);
    }


    /**This produces an example polyomial on which to
       test the various subdivision operations*/

    public Poly5 examplePoly() {
	Poly6 P=DataCM.f();
	int[][] b=ModuliSpace.simplex3(0);
	Poly5 S=P.toCube(b);
	return(S);
    }

    public void polynomialDilate() {
	Poly5 P=examplePoly();
	  Poly5 Q=P.dilate();
	  int[] x1=random5(10);
	  int[] x2={x1[0],x1[1],x1[2],x1[3],x1[4]};
	  x1[0]=2*x1[0];
	  BigInteger a=P.evaluate(x2);
	  BigInteger b=Q.evaluate(x1);
	  int m=P.exponentLimit(0);
	  m=(int)(Math.pow(2,m));
	  BigInteger cc=Mono5.copy(m);
	  a=a.multiply(cc);
	  System.out.println("----");
	  System.out.println(a.toString());
	  System.out.println(b.toString());
    }

    public void polynomialRotate() {
	Poly5 P=examplePoly();
	  Poly5 Q=P.rotate(2);
	  int[] x1=random5(10);
	  int[] x2={x1[2],x1[3],x1[4],x1[0],x1[1]};
	  BigInteger a=P.evaluate(x2);
	  BigInteger b=Q.evaluate(x1);
	  System.out.println("----");
	  System.out.println(a.toString());
	  System.out.println(b.toString());
    }

    public void polynomialReflect() {
	Poly5 P=examplePoly();
	  Poly5 Q=P.reflect();
          int[] x1=random5(20);
	  int[] x2={1-x1[0],x1[1],x1[2],x1[3],x1[4]};
	  BigInteger a=P.evaluate(x2);
	  BigInteger b=Q.evaluate(x1);
	  System.out.println(a.toString());
	  System.out.println(b.toString());
    }

    public void signEvaluator() {
	Poly6 f= DataCM.f();
	HALT=false;
	int count=0;
	while(HALT==false) {
	    int[] x=random6(20);
	    BigInteger[] xx=new BigInteger[6];
	    for(int i=0;i<6;++i) {
		Integer temp=new Integer(x[i]);
		xx[i]=new BigInteger(temp.toString());
	    }
	    int k=random(20);
	    int s=f.evaluateSign(xx);
	    double[] d=new double[6];
	    for(int i=0;i<6;++i) d[i]=x[i]/Math.pow(2.0,k);
	    double t=CayleyMenger.f(d);
	    if(count%1000==0) System.out.println("success "+count);
	    if(s*t<0) {
		System.out.println("fail");
		System.out.println(1/0);
	    }
	    ++count;
	}
    }

    public void misc() {
	double[] a={6,3,3,3,3,6};
	double y=CayleyMenger.f(a);
	int[] q={1,1,1,1,1,1};
	double z=CayleyMenger.df(q,a);
	System.out.println(y+" "+z);
    }



}


