import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.math.*;


public class Poly6 {
    Mono6[] m=new Mono6[50000];
    int count;

    public Poly6() {}

    public Poly6(Poly6 P) {
	this.count=P.count;
	for(int i=0;i<this.count;++i) this.m[i]=new Mono6(P.m[i]);
    }

    public Poly6(int[][] a) {
	this.count=a.length;
	for(int i=0;i<this.count;++i) {
	    int c=a[i][0];
	    int[] b={a[i][1],a[i][2],a[i][3],a[i][4],a[i][5],a[i][6]};
	    this.m[i]=new Mono6(c,b);
	}
    }

    public BigInteger evaluate(int[] x) {
	BigInteger a=new BigInteger("0");
	for(int i=0;i<count;++i) {
	    a=a.add(m[i].evaluate(x));
	}
	return(a);
    }

    public BigInteger evaluate(BigInteger[] x) {
	BigInteger a=new BigInteger("0");
	for(int i=0;i<count;++i) {
	    a=a.add(m[i].evaluate(x));
	}
	return(a);
    }

    /**This computes the sign of a Poly6 on a 10-adic rational
       The rational is (a[0]/10^k,...,a[6]/10^k)**/

    public int evaluateSign(BigInteger[] x) {
	BigInteger y=evaluate(x);
	BigInteger Z=new BigInteger("0");
	return(y.compareTo(Z));
    }


    public int exponentLimit(int k) {
	int max=0;
	for(int i=0;i<count;++i) {
	    if(max<m[i].e[k]) max=m[i].e[k];
	}
	return(max);
    }

    public int[] exponentLimit() {
	int[] a=new int[6];
	for(int i=0;i<6;++i) a[i]=exponentLimit(i);
	return(a);
    }

    public static BigInteger copy(int a) {
	Integer A=new Integer(a);
	BigInteger AA=new BigInteger(A.toString());
	return(AA);
    }

    public static Poly6 random() {
	Poly6 P=new Poly6();
	P.count=5;
	for(int i=0;i<5;++i) P.m[i]=Mono6.random();
	P=CompressPoly.main(P);
	return(P);
    }

    public static Poly6 times(Poly6 P,Poly6 Q) {
	Poly6 R=new Poly6();
	int c=0;
	for(int i=0;i<P.count;++i) {
	    for(int j=0;j<Q.count;++j) {
		R.m[c]=Mono6.times(P.m[i],Q.m[j]);
		++c;
	    }
	}
	R.count=c;
	R=CompressPoly.main(R);
	return(R);
    }

    public static Poly6 plus(Poly6 P,Poly6 Q) {
	Poly6 R=new Poly6();
	int c=0;
	for(int i=0;i<P.count;++i) {
	    R.m[c]=new Mono6(P.m[i]);
	    ++c;
	}

	for(int i=0;i<Q.count;++i) {
	    R.m[c]=new Mono6(Q.m[i]);
	    ++c;
	}
	R.count=c;
	R=CompressPoly.main(R);
	return(R);
    }

    /**creates a linear polynomial*/

    public static Poly6 linear(int[] a) {
	Poly6 P=new Poly6();
	int c=0;
	int[] z={0,0,0,0,0,0};
	for(int i=0;i<6;++i) {
	    if(a[i]!=0) {
	       P.m[c]=new Mono6(a[i],z);
	       P.m[c].e[i]=1;
	       ++c;
	    }
	}
	P.count=c;
	return(P);
    }

    /**scales a polynomial*/

    public Poly6 scale(int n) {
	Integer n1=new Integer(n);
	BigInteger n2=new BigInteger(n1.toString());
	Poly6 A=constant(n2);
	A=times(A,this);
	return(A);
    }


    /**Creates a constant polynomial*/

    public static Poly6 constant(BigInteger n) {
	Poly6 I=new Poly6();
	I.count=1;
	int[] a={0,0,0,0,0,0};
	I.m[0]=new Mono6(n,a);
	return(I);
    }



    /**Here is the main transformation routine.  It produces
       the map F: V-->Y where V is the unit cube in R^5 and
       V is a 5-simplex in R^6.  The data (a) encodes V.
       The vertices of Y are a[0],...,a[5]*/

    public Poly5 toCube(int[][] a) {
	Poly6 P=this.transform1(a);
	Poly6 Q=P.transform2();
	Poly5 R=Q.transform3();
	return(R);
    }

    /**Let X be the symmetric 5-simplex, the convex hull of the
       standard basis vectors in R^6.  Let Y be some other 5-simplex
       in R^6, given as the convex hull of basis vectors.  Let
       A: X -> Y be the linear transformation taking X to Y which
       carries the standard basis to the basis for Y.  Let P be a
       polynomial.  This routine computes the polynomial PA, which
       we think of as acting on X.*/


    public Poly6 transform1(int[][] a) {
	Poly6 A=new Poly6();
	for(int i=0;i<this.count;++i) {
	    Poly6 B=transform1(a,this.m[i]);
	    A=plus(A,B);
	}
	return(A);
    }

    public static Poly6 transform1(int[][] a,Mono6 M) {
	Poly6[] A=new Poly6[6];
	for(int i=0;i<6;++i) {
	    int[] I={a[0][i],a[1][i],a[2][i],a[3][i],a[4][i],a[5][i]};
	    A[i]=linear(I);
	}
	return(transform1(A,M));
    }

    public static Poly6 transform1(Poly6[] A,Mono6 M) {
	Poly6 I=constant(M.c);
	for(int i=0;i<6;++i) {
	    for(int j=0;j<M.e[i];++j) {
		I=times(I,A[i]);
	    }
	}
	return(I);
    }

    /**Let W be the basic simplex, cross 0, sitting in R^6.  Let
       X be the symmetric simplex.  Let L: W -> X be the affine map
       carrying W to X.  This routine computes PL.*/

    public Poly6 transform2() {
	Poly6 A=new Poly6();
	for(int i=0;i<this.count;++i) {
	    Poly6 B=transform2(this.m[i]);
	    A=plus(A,B);
	}
	return(A);
    }

    public static Poly6 transform2(Mono6 M) {
	Poly6[] A=new Poly6[6];
	A[0]=constant(new BigInteger("1"));
	int[] e={1,0,0,0,0,0};
	A[0].m[1]=new Mono6(-1,e);
	A[0].count=2;
	int[] k1={1,-1,0,0,0,0};
	A[1]=linear(k1);
	int[] k2={0,1,-1,0,0,0};
	A[2]=linear(k2);
	int[] k3={0,0,1,-1,0,0};
	A[3]=linear(k3);
	int[] k4={0,0,0,1,-1,0};
	A[4]=linear(k4);
	int[] k5={0,0,0,0,1,0};
	A[5]=linear(k5);
	return(transform1(A,M));
    }



    /**Let V be the unit cube in R5 and let W be the
       basic simplex, cross 0, sitting in R6.  Let F be
       the map (a,b,c,d,e)->(a,ab,abc,abcd,abcde,0).
       Then F: V->W.  This computes PF.*/


    public Poly5 transform3() {
	Poly5 A=new Poly5();
	A.count=this.count;
	for(int i=0;i<this.count;++i) {
	    A.m[i]=transform3(this.m[i]);
	}
	return(A);
    }

    public static Mono5 transform3(Mono6 A) {
	Mono5 B=new Mono5();
	B.c=new BigInteger(A.c.toString());
	B.e[0]=A.e[0]+A.e[1]+A.e[2]+A.e[3]+A.e[4];
	B.e[1]=A.e[1]+A.e[2]+A.e[3]+A.e[4];
	B.e[2]=A.e[2]+A.e[3]+A.e[4];
	B.e[3]=A.e[3]+A.e[4];
	B.e[4]=A.e[4];
	return(B);
    }


    /**printouts*/
    public void print(String NAME) {
	System.out.print(NAME+"={");
	for(int i=0;i<count;++i) {
	    m[i].print();
	    if(i<count-1) System.out.print(",");
	    if(i==count-1) System.out.print("};");
	}
	System.out.println("");
    }

    public void print(Output OUT,String NAME) {
	OUT.print(NAME+"={");
	for(int i=0;i<count;++i) {
	    m[i].print(OUT);
	    if(i<count-1) OUT.print(",");
	    if(i==count-1) OUT.print("};");
	}
	OUT.println("");
    }

}